@@ -192,6 +192,7 @@ impl<'a, I: Tokens> Parser<I> {
         if !self.syntax().decorators() {
             return Ok(vec![]);
         }
+        trace_cur!(self, parse_decorators);
 
         let mut decorators = vec![];
         let start = cur_pos!(self);
@@ -221,6 +222,7 @@ impl<'a, I: Tokens> Parser<I> {
 
     fn parse_decorator(&mut self) -> PResult<Decorator> {
         let start = cur_pos!(self);
+        trace_cur!(self, parse_decorator);
 
         assert_and_bump!(self, '@');
 
@@ -229,24 +231,12 @@ impl<'a, I: Tokens> Parser<I> {
             expect!(self, ')');
             expr
         } else {
-            let mut expr = self
+            let expr = self
                 .parse_ident(false, false)
                 .map(Expr::from)
                 .map(Box::new)?;
 
-            while eat!(self, '.') {
-                let ident = self.parse_ident(true, true)?;
-
-                let span = Span::new(start, expr.span().hi(), Default::default());
-
-                expr = Box::new(Expr::Member(MemberExpr {
-                    span,
-                    obj: expr,
-                    prop: MemberProp::Ident(ident),
-                }));
-            }
-
-            expr
+            self.parse_subscripts(Callee::Expr(expr), false, true)?
         };
 
         let expr = self.parse_maybe_decorator_args(expr)?;
